﻿using Microsoft.Practices.Unity;
using Prism.Commands;
using Prism.Logging;
using Prism.Mvvm;
using SyncfusionDockingDispose.Views;
using System.Collections.ObjectModel;
using System.Windows;
using System;

namespace SyncfusionDockingDispose.ViewModels
{
    public class MainWindowViewModel : ViewModelBase
    {
        public DelegateCommand AddContentCommand { get; private set; }

        public DelegateCommand OpenContentCommand { get; private set; }

        public DelegateCommand CloseAllCommand { get; private set; }

        public DelegateCommand ForceGCCommand { get; private set; }

        private string _title = "Prism Unity Application";
        public string Title
        {
            get { return _title; }
            set { SetProperty(ref _title, value); }
        }

        public ObservableCollection<ViewModelBase> DockWindows { get; private set; }

        private ViewModelBase activeDocument;
        public ViewModelBase ActiveDocument
        {
            get { return activeDocument; }
            set { SetProperty(ref activeDocument, value); }
        }

        private IUnityContainer container;

        public MainWindowViewModel(ILoggerFacade logger, IUnityContainer container) : base(logger)
        {
            this.container = container;
            DockWindows = new ObservableCollection<ViewModelBase>();

            AddContentCommand = new DelegateCommand(() =>
            {
                var vm = container.Resolve<ContentPageViewModel>();
                vm.RequestClose += close; // () => DockWindows.Remove(vm);
                DockWindows.Add(vm);
            });
            OpenContentCommand = new DelegateCommand(() =>
            {
                openContentWindow();
            });
            CloseAllCommand = new DelegateCommand(() =>
            {
                while (DockWindows.Count > 0)
                    DockWindows[0].OnRequestClose();
            });
            ForceGCCommand = new DelegateCommand(() =>
            {
                GC.Collect();
            });
        }

        /// <summary>
        /// This is a bit hacked since we don't know which VM was closed.  We will assume that the oldest window
        /// is always the window we are closing.  I don't think that the generic handler commented out above 
        /// is what is anchoring us from GC, but i figured it was worth removing any doubt.  Once proper GC
        /// is being performed, i will test with the genric function to see if we still work.
        /// </summary>
        private void close()
        {
            var mv = DockWindows[0]; 
            DockWindows.Remove(mv);
            mv.RequestClose -= close;
        }

        private void openContentWindow()
        {
            Window window = new Window();
            var vm = container.Resolve<ContentPageViewModel>();
            vm.RequestClose += () => window.Close();
            var view = container.Resolve<ContentPage>();
            view.DataContext = vm;
            window.Content = view;
            window.Closed += (s, o) =>
            {
                view.DataContext = null;
            };
            window.Show();
        }
    }
}
